/* 
load_foreign_library('.../dltest.so').
*/
#include "SWI-Prolog.h"

char* C_pref="--from C:";

/*****************************************************************
 *  say_hello: simple deterministic function to say hello from C 
 *****************************************************************/

static foreign_t
pl_say_hello(term_t to)
{ char *a;

  if ( PL_get_atom_chars(to, &a) )
  { 
    Sprintf("%s hello to %s\n", C_pref, a);
    PL_succeed;
  }

  PL_fail;
}

/*****************************************************************
 *  natural_number_below_n/2 (i,o): 
 *  nondeterministic function to generate natural numbers bellow N 
 *****************************************************************/


typedef struct                  /* define a context structure */
{ 
  int last_n;
} context;

foreign_t
pl_natural_number_below_n(term_t N, term_t N1, foreign_t handle)
{ context * ctxt;
  char* s; 

  switch( PL_foreign_control(handle) )
  { case PL_FIRST_CALL:
        ctxt = (context*) malloc(sizeof(context));
        if (PL_get_integer(N, &(ctxt->last_n))){
	  Sprintf("%s first call (N=%d)\n", C_pref, ctxt->last_n);
	  ctxt->last_n--;
	  if (ctxt->last_n<0) { free(ctxt); PL_fail;}
	  else
	  if (PL_unify_integer(N1, ctxt->last_n))
	    PL_retry_address(ctxt);
	  else { free(ctxt); PL_fail;}
	}
	else {
	    PL_get_chars(N, &s, CVT_ALL);
	    Sprintf("%s wrong argument %s\n", C_pref, s);
	    free(ctxt); PL_fail;
	}
        
    case PL_REDO:
        ctxt = PL_foreign_context_address(handle);
	Sprintf("%s redo call (prev N1=%d)\n", C_pref, ctxt->last_n);
	ctxt->last_n--;
	if (ctxt->last_n<0) { free(ctxt); PL_fail;}
	else
	if (PL_unify_integer(N1, ctxt->last_n))
	  PL_retry_address(ctxt);
	else { free(ctxt); PL_fail;}

    case PL_CUTTED:
        ctxt = PL_foreign_context_address(handle);
        Sprintf("%s Cutted\n", C_pref);
        free(ctxt);
        PL_succeed;
  }
}



install_t
install()
{ PL_register_foreign("say_hello", 1, pl_say_hello, 0);
  PL_register_foreign("natural_number_below_n", 2, pl_natural_number_below_n, 
		      PL_FA_NONDETERMINISTIC);  
}


